//
//  ArteMisya.h
//  ArtesemaOGL
//
//  Created by touta on 06/12/11.
//  Copyright 2006 TSBps. All rights reserved.
//

/*!
    @header ArteMisya
 @abstract   ArteMisya, implements of Misya at proj.Artesema
 @discussion
	ArteMisya is "model class" and work with some "view class"(viewer).
	ArteMisya has no method to display inner infomation about SO,
	and viewer will display ScreenObject structure contained in ArteMisya.
 
 <pre>
   Misya
   |-screens(MArray)
     |-currentScreen(MArray)
     | |-so_object(MDic)
     | |  __obj:image
     | |  
     | |-so_object(MDic)
     | |-screeninfo(MDic)
     | |  __obj:__info
     | |  focusing:-1
     |
     |-backyard_screen(MArray)
     | |-so_object(MDic)
   
   |-scenario(MArray)
   	|-scriptcode(String)
   	|-scriptcode(String)
   	|-...
 </pre>
*/


#import <Foundation/Foundation.h>
#if TARGET_OS_IPHONE
#import <CoreGraphics/CoreGraphics.h>
#elif TARGET_OS_MAC
#import <QuartzCore/QuartzCore.h>
#endif

#import "ArteMisyaDelegate.h"
	//Delegate for expanding Misya

#import "ArteMisyaViewerProtocol.h"
	//Protocol with Viewer

#import "ArteMisyaUtils.h"
	//Util functions of ArteMisya


@interface ArteMisya : NSObject <NSCoding>{
@protected
	id <ArteMisyaDelegate> delegate;
	id <ArteMisyaViewerProtocol> viewer;
	
	NSMutableArray *screens;
	NSMutableArray *scenario;
	int focusing;
	BOOL skippable;

@public
	BOOL debug;
}

- (void)setViewer:(id <ArteMisyaViewerProtocol>)obj;
	// viewer is object for display ScreenObject.
- (void)setDelegate:(id <ArteMisyaDelegate>)obj;
- (void)setDebug:(BOOL)bl;
- (BOOL)skippable;

// DO NOT Edit Follow mutables.
//	(except viewer to add infomation for displaying)
- (NSMutableArray *)screens;
- (NSMutableArray *)scenario;
//	To edit screen or scenario, use method like addObject: or removeObject:.


#pragma mark Misya-part
- (void)reset;

#pragma mark ShiaGS-part
- (NSString *)call:(NSString *)path withArg:(NSDictionary *)arg;



//** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **//
//*** *** *** *** *** *** *** *** *** *** *** *** *** *** ***//
//**** **** **** **** **** **** **** **** **** **** **** ****//
//*** *** *** *** *** *** *** *** *** *** *** *** *** *** ***//
//** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **//

#pragma mark -
#pragma mark Screen Manage
// Screen Manage
//	Misya keep some screens in stack

// add
- (void)addNewScreen;
- (void)addNewScreenWithObjects:(NSArray *)ar;
- (void)addNewScreenNamed:(NSString *)name withObjects:(NSArray *)ar;

// remove
- (void)removeCurrentScreen;
- (void)removeAllScreens;

// status
- (void)saveCurrentScreenStatus;
// save current status to info object
- (void)revertCurrentScreenStatus;
// revert status from info object
- (id)getInfoObjAtScreen:(NSMutableArray *)scr;
// get info object for screen

- (NSMutableArray *)currentScreen;
//current screen

#pragma mark Object Manage
// Manage objects in current screen.
- (void)addObjects:(NSArray *)ar;
- (void)addObject:(NSDictionary *)obj;

- (void)removeAllObjects;
- (void)removeObjectNamed:(NSString *)name;

- (void)setValue:(NSString *)value toGroup:(NSString *)grp forKey:(NSString *)key;
- (void)setValue:(NSString *)value toObjects:(NSMutableArray *)targets forKey:(NSString *)key;
- (void)setValue:(NSString *)value toObject:(NSMutableDictionary *)target forKey:(NSString *)key;

- (id)objectNamed:(NSString *)name;
- (id)objectNamed:(NSString *)name ofGroup:(NSString *)grp;
- (int)indexOfObjectNamed:(NSString *)name;
- (id)objectsOfGroupNamed:(NSString *)grp;

#pragma mark Load from File
// load from file
//	load objects from TSBff/SO or XML format file.
//
// add objects from file
- (void)addScreenFromFileNamed:(NSString *)sofile;
- (void)addObjectsFromFileNamed:(NSString *)sofile;
// get objects from file
- (NSArray *)getScreenObjectsFromFileNamed:(NSString *)name;
- (NSArray *)getScreenObjectsFromSOFileAtPath:(NSString *)filepath;
- (NSArray *)getScreenObjectsFromXMLFileAtPath:(NSString *)filepath;

#pragma mark Focus Manage
// Manage focus
//	1 screen have 1 focus.
//	focus means selecting in Misya/ScrenObject.
//	There are some way to focusing.
//	See [ShiaDoc/Misya/Misya focusing]
// 2D focus management
- (BOOL)focusRightOne;
- (BOOL)focusLeftOne;
- (BOOL)focusUpperOne;
- (BOOL)focusDownerOne;
- (BOOL)focusNextWithKey:(NSString *)key;
// 1D focus management
- (BOOL)focusNextSelectable;
- (BOOL)focusPrevSelectable;
- (int)focusing;

- (BOOL)isSelectedObject:(NSDictionary *)obj;
- (BOOL)isSelectableObject:(NSDictionary *)obj;
- (BOOL)isPointableObject:(NSDictionary *)obj;
- (BOOL)isVisibleObject:(NSDictionary *)obj;
- (CGRect)rectOfObject:(NSDictionary *)obj;

#pragma mark focus event
- (BOOL)runFocusing;
- (BOOL)moveFocusing:(int)newFocusing;



//** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **//
//*** *** *** *** *** *** *** *** *** *** *** *** *** *** ***//
//**** **** **** **** **** **** **** **** **** **** **** ****//
//*** *** *** *** *** *** *** *** *** *** *** *** *** *** ***//
//** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **//

#pragma mark -
#pragma mark SOScript

// order stack
- (BOOL)loadOrdersFromFileNamed:(NSString *)sce;
- (BOOL)loadOrdersFromFileAtPath:(NSString *)path;
- (BOOL)execOrder;

- (void)pushOrder:(NSString *)order;
- (void)pushOrders:(NSArray *)orders;
- (void)queueOrder:(NSString *)order;
- (void)queueOrders:(NSArray *)orders;
- (void)clearOrderQueue;

// execuing script in step by step
- (BOOL)execScript:(NSString *)scriptcode withTarget:(NSMutableDictionary *)target;
- (BOOL)execScriptForKey:(NSString *)key ofTarget:(NSMutableDictionary *)target;
- (NSString *)execScriptCode:(NSString *)code withTarget:(NSMutableDictionary *)target;
- (NSString *)execScriptCodeOfSet:(NSString *)code withTarget:(NSMutableDictionary *)target;
- (NSString *)execScriptCodeOfGet:(NSString *)code withTarget:(NSMutableDictionary *)target;
- (NSString *)execScriptCodeOfSysCall:(NSString *)code withTarget:(NSMutableDictionary *)target;
- (NSString *)execScriptCodeOfExec:(NSString *)code withTarget:(NSMutableDictionary *)target;
- (NSString *)execScriptCodeOfShiaCall:(NSString *)code withTarget:(NSMutableDictionary *)target;





@end

