//
//  ArteOGLImageTexture.h
//  ArtesemaOGL
//
//  Created by touta on 06/03/23.
//  Copyright 2006 TSBps. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <OpenGL/OpenGL.h>
#import <OpenGL/gl.h>
#import <OpenGL/glext.h>

#import "ArteOGLObject_resource.h"


@interface ArteOGLImageTexture : NSObject {
	BOOL texturized;
	NSBitmapImageRep *bmprep;
	NSSize imgsize;
	CGSize texsize;
	GLuint texid;
	NSString *imgname; //name of image, use for human level identification
}

#pragma mark initialize
- (id)initWithImageNamed:(NSString *)name;
- (id)initWithImageNamed:(NSString *)name withExtension:(NSString *)ext;
- (id)initWithFrameworkImageNamed:(NSString *)name;
- (id)initWithNSImage:(NSImage *)img;
- (id)initWithBitmapImageRep:(NSBitmapImageRep *)rep;


#pragma mark set bmprep
// Normally, there's no need to care about bmprep.
//	You also can init texture object by -init and set bmprep with follow methods,
//	but it is not recommended.
- (BOOL)setImageAtPath:(NSString *)imagepath;
- (BOOL)setNSImage:(NSImage *)img;
- (BOOL)setBitmapImageRep:(NSBitmapImageRep *)rep;


#pragma mark texturize
// Normally, there's no need to care about texturize.
//	We only call -texid to and get its texture id.
//	Or for drawing, only call -OGLdrawAtPoint: and -OGLdrawAtRect:alpha: is enough after initialize.
- (BOOL)texturize;
	// return YES, already texturized or sucess to createTexture.
	//	otherwise, can not ready texture, return NO.
- (BOOL)createTexture;
	// create texture of image from bmprep
- (void)destructTexture;
	// destruct texture
- (BOOL)isTextureResident;
	// Is texture exist on memory?
	//	see [ glAreTexturesResident ]

#pragma mark draw
// Easy drawing.
//	some object may call this method to draw image in their turn.
- (void)OGLdrawAtPoint:(CGPoint)pt;
- (void)OGLdrawAtPoint:(CGPoint)pt fraction:(float)alpha;
- (void)OGLdrawInRect:(CGRect)rect fraction:(float)alpha;

#pragma mark values

- (NSSize)		imgsize;
	// size of image
- (NSString *)	imgname;
	// name of image
- (CGSize)		texsize;
	// size of texture
- (GLuint)		texid;
	// return texture-id.
	//	if no texture, create it.

@end
