//
//  ArteOGLObject.h
//  ArtesemaOGL
//
//  Created by touta on 06/02/26.
//  Copyright 2006 TSBps. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <OpenGL/OpenGL.h>
#import <OpenGL/gl.h>
#import <OpenGL/glext.h>

#import "ArteOGLDefines.h"
#import "ArteOGLViewProtocol.h"

#import "ArteOGLOLE.h"
@class ArteObjectBehavior;

// ArteOGL Object
//
//	Root class of object use in ArteOGL system.
//	Contains basical behavior of ArteOGL.
//

@interface ArteOGLObject : NSObject {
	int coordmode; //ArteOGLObject-CoordMode
	
	//COORD
	CGPoint texvirtual; //vx,vy		**based with [arte-origin] or [dif of moving]
	CGPoint texpoint;	//x,y		**always based with view-origin
	float texzdepth;	//z
	float texalpha;		//flaction
	int texblend;		//blend mode
	
	//MOVING
	CGPoint texspeed;	//sx,sy
	CGPoint texaccela;	//ax,ay
	CGPoint texforce;	//fx,fy
	
	//ROTATING
	float rotatedegree;
	CGPoint rotateorigin;
	float rotatespeed;
	float rotateaccela;
	
	//SIZE
	CGSize texsize;		//width,height
	CGRect atkrect;		//ATtacK-RECT
	CGRect exirect;		//EXIst-RECT
	
	//STATUS
	int arteclass;		//
	int arteteam;		//	
	int lifecount;		// life (frame)count
	int extralife;		// lifetime after dead
	int scorepoint;		//
	
	//FLAGS
	BOOL attackFlag;	// Object send Hit-Check
		//YES: attack with Hit-Check
	BOOL targetFlag;	// Object recieve Hit-Check
		//YES: attacked from someone with Hit-Check
	BOOL physicalFlag;	// Object is caluculated in Physical-Check
		//YES: push/pushed with other object with Pysic.
	BOOL staticFlag;	// Object never move.
		//YES: never move.
	BOOL livingFlag;	// Still living or zombie for waiting
		//YES: Keep alive!
		//NO : Will remove in soon.
	
	//STATE
	BOOL contiguous[4]; 
	
	ArteObjectBehavior *behavior;
	// OLE is replaced with ArteObjectBehavior
	BOOL useOLE; // DEPRECATED //
	ArteOGLObjectOLE OLE; // DEPRECATED //
}


#pragma mark Coordinates
- (void)setCoordMode:(int)mode;
- (int)coordMode;

// v texpoint v
- (void)setX:(float)x y:(float)y z:(float)z;
- (void)setXY:(CGPoint)p;
- (void)setX:(float)x;
- (void)setY:(float)y;
- (void)setZ:(float)z;
- (void)setScreenXY:(CGPoint)sp;
- (CGPoint)xy;
- (float)x;
- (float)y;
- (float)z;
// v texvirtual v
- (void)setVirtualXY:(CGPoint)vp;
- (void)setVirtualX:(float)vx;
- (void)setVirtualY:(float)vy;
- (void)addXY:(CGPoint)p;
- (void)addX:(float)x;
- (void)addY:(float)y;
- (void)addZ:(float)z;
- (CGPoint)vxy;
- (float)vx;
- (float)vy;

#pragma mark Size
- (void)setSize:(CGSize)s;
- (void)setWidth:(float)w;
- (void)setHeight:(float)h;
- (CGSize)size;
- (float)width;
- (float)height;
- (void)scaleSize:(float)s;

- (void)setFraction:(float)f;
- (float)fraction;
- (void)setBlendMode:(ArteOGLViewBlendMode)mode;
- (float)blendMode;

#pragma mark Speed
- (void)setSpeed:(CGPoint)p;
- (void)setSpeedX:(float)x;
- (void)setSpeedY:(float)y;
- (CGPoint)addSpeed:(CGPoint)p;
- (float)addSpeedX:(float)x;
- (float)addSpeedY:(float)y;
- (CGPoint)speed;

#pragma mark Accela
- (void)setAccela:(CGPoint)p;
- (void)setAccelaX:(float)x;
- (void)setAccelaY:(float)y;
- (CGPoint)addAccela:(CGPoint)p;
- (float)addAccelaX:(float)x;
- (float)addAccelaY:(float)y;
- (CGPoint)accela;

#pragma mark Force
- (void)setForce:(CGPoint)p;
- (void)pushToX:(float)x;
- (void)pushToY:(float)y;
- (CGPoint)force;

#pragma mark Rotate
-(void)setDegree:(float)deg;
-(float)degree;
-(void)setRadian:(float)rad;
-(float)radian;
-(void)setRotateOrigin:(CGPoint)pt;
-(CGPoint)rotateOrigin;
-(void)setRotateSpeed:(float)rpf;
-(float)rotateSpeed;

#pragma mark Area
-(void)setAttackArea:(CGRect)r;
-(void)setAttackAreaFromOwnSize;
-(void)setExistArea:(CGRect)r;
-(void)setExistAreaFromOwnSize;

#pragma mark setProperty
- (void)setProperty:(NSDictionary *)dic;
	- (void)setBasicProperty:(NSDictionary *)dic;
	- (void)setAdvancedProperty:(NSDictionary *)dic;
	- (void)setOLEProperty:(NSDictionary *)dic; // DEPRECATED //

#pragma mark Behavior
- (void)setBehavior:(ArteObjectBehavior *)behavior;
- (void)addBehavior:(ArteObjectBehavior *)behavior;

#pragma mark Class/Team/Life
- (void)setArteClass:(int)c;
- (int)arteClass;
- (void)setArteTeam:(int)t;
- (int)arteTeam;
- (void)setLifecount:(int)t;
- (int)lifecount;

#pragma mark flag/rect
- (BOOL)attackFlag;
- (BOOL)targetFlag;
- (BOOL)livingFlag;
- (BOOL)physicalFlag;
- (BOOL)staticFlag;
- (CGRect)attackRect;
- (CGRect)existRect;
- (CGRect)movingRange;
- (int)score;


#pragma mark Calc/Push/Draw
// Calculation
- (void)OGLconvert:(id <ArteOGLViewProtocol>)arteview; //convert virtual cooord before calculation
- (void)OGLcalc:(id <ArteOGLViewProtocol>)arteview; //main calcuration method. call from ArteOGLView
//	- (void)OGLcalcOLE:(id <ArteOGLViewProtocol>)arteview; //sub-method  Object Level Effect. // DEPRECATED //

// HitCheck/PhysicCheck
- (void)OGLhit:(id <ArteOGLViewProtocol>)arteview by:(ArteOGLObject *)obj;
- (CGPoint)OGLpush:(CGPoint)pushwidth;

// Drawing
- (void)OGLdraw:(id <ArteOGLViewProtocol>)arteview; //main drawing method, call from ArteOGLView


#pragma mark action
- (void)die; //
- (void)kill; //kill by someone. dead immediate.

#pragma mark event
- (void)didDie;


#pragma mark description
- (NSString *)shortDescription;
- (NSString *)descriptionMessage;
@end


#import "ArteOGLObject_resource.h"
