//
//  ArteOGLString.h
//  ArtesemaOGL
//
//  Created by touta on 06/08/02.
//  Copyright 2006 TSBps. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "ArteOGLObject.h"
#import "ArteOGLStringTexture.h"
#import <OpenGL/OpenGL.h>
#import <OpenGL/gl.h>
#import <OpenGL/glext.h>

@interface ArteOGLString : ArteOGLObject {
	ArteOGLStringTexture * texture;
	BOOL stringfade;
	int stringfadeCount;
	float stringfadeValue[4];
}


#pragma mark Texture Cache Manage
// cache texture
//	Follow methods create and texture with arguments and add to cache list.
//	Cached texture is referable only with its name.
+ (BOOL)cacheTextureWithAttributedString:(NSAttributedString *)as
								 forName:(NSString *)name;
+ (BOOL)cacheTextureWithSystemFontString:(NSString *)str
							   grayscale:(float)gs
									size:(int)fs
								 forName:(NSString *)name;
+ (BOOL)cacheTextureWithString:(NSString *)str
					 grayscale:(float)gs
					  fontname:(NSString *)fn
					  fontsize:(int)fs
					   forName:(NSString *)name;

// create and cache texture with string
// return cached texture
//	Follow methods check cache list with given name first.
//	Then if not exist, create texture with arguments and add to cache list.
//	After all, return texture.
+ (id)cachedTextureWithAttributedString:(NSAttributedString *)as forName:(NSString *)name;
+ (id)cachedTextureWithSystemFontString:(NSString *)str forName:(NSString *)name;
+ (id)cachedTextureWithBlackSystemFontString:(NSString *)str forName:(NSString *)name;
+ (id)cachedTextureWithWhiteSystemFontString:(NSString *)str forName:(NSString *)name;
+ (id)cachedTextureWithString:(NSString *)str
					grayscale:(float)gs			// 0.0-1.0; 1.0:white, 0.0:black
					 fontname:(NSString *)fn	// font name as used in PostScript language code
					 fontsize:(int)fs			// fontsize; if (fs<=0) fs = systemFontSize;
					  forName:(NSString *)name;


+ (ArteOGLStringTexture *)sharedTextureNamed:(NSString *)name;
// get share-managed texture
//	Some object use this method to get string-texture to custom draw in their turn.

+ (void)uncacheTextureForName:(NSString *)name;

+ (void)clearSharedTextures;
// clear cache
//	User should call this method to clear memory.
//	Better to call at proper time.


#pragma mark initialize

+ (id)objectWithAttributedString:(NSAttributedString *)name;
+ (id)objectWithString:(NSString *)str attributes:(NSDictionary *)dic;
+ (id)objectWithString:(NSString *)str
			 grayscale:(float)gs		// 0.0-1.0; 1.0:white, 0.0:black
			  fontsize:(int)fs;			// systemFontSize += n
// get string object.
//	Normally, call this method to get image object.
//	
//	Notice: Returned object is NOT cached.
//	Because string has less reusability than image.
//	If you want to cache it, 

- (id)initWithAttributedString:(NSAttributedString *)str;
- (id)initWithString:(NSString *)str attributes:(NSDictionary *)att;
- (id)initWithProperty:(NSDictionary *)prop;

#pragma mark texture manage
- (void)switchTexture:(id)texobj;
- (void)changeString:(NSString *)str attributes:(NSDictionary *)dic;
- (void)setColor:(NSColor *)col;

- (void)startStringfade;
//- (void)OGLdraw;	//override

- (void)ensureTexturize;

@end
