/* ArteOGLView */

#import <objc/objc.h>

#import <Cocoa/Cocoa.h>
#import <CoreVideo/CoreVideo.h>

#import <OpenGL/OpenGL.h>
#import <OpenGL/gl.h>
#import <OpenGL/glext.h>
#import <OpenGL/glu.h>

#import "ArteOGLViewProtocol.h"
#import "ArteOGLViewDelegateProtocol.h"
#import "ArteOGLDefines.h"

#import "ArteOGLObject.h"


typedef enum{
	ArteOGLViewProcessScroll	= 0x1<<0,
	ArteOGLViewProcessConvert	= 0x1<<1,
	ArteOGLViewProcessCalc		= 0x1<<2,
	ArteOGLViewProcessHitCheck	= 0x1<<3,
	ArteOGLViewProcessDraw		= 0x1<<4,
	ArteOGLViewProcessAll = ArteOGLViewProcessScroll |
							ArteOGLViewProcessConvert |
							ArteOGLViewProcessCalc |
							ArteOGLViewProcessHitCheck |
							ArteOGLViewProcessDraw
} ArteOGLViewProcessFlags;

@interface ArteOGLView : NSOpenGLView <ArteOGLViewProtocol>{
	IBOutlet id <ArteOGLViewDelegateProtocol> controller;
	NSTimer *timer;
	CVDisplayLinkRef displayLink;
	BOOL oglinited;
	BOOL FullScreenMode;
	NSOpenGLContext *fullScreenContext;
	NSWindow *originalWindow;
	NSView *originalSuperview;
	NSRect originalFrame;
	BOOL originalHideOnDeactive;
	CGRect screenRect;
	
	BOOL pause;
	BOOL pauseDrawed;
	
	//count for timer
	long framecount;
	int processflag;
	
	//scrolling
	CGPoint vcoord;
	int scrollMode[2]; //[x,y]
	float scrollSpeed[2]; //[x,y]
	float scrollEdge[4]; //[left,bottom,right,top]
	float scrollLimit[4]; //[left,bottom,right,top]
	float scrollTarget[2]; //x,y
	float scrollDelay[2]; //x,y
	BOOL stageUse[4]; //[left,bottom,right,top]
	float stageBorder[4]; //[left,bottom,right,top]
	
	//FrameRates
	long fps;
	BOOL FPSrunning;
	NSTimer *FPStimer;
	long FPSlastframe;
	long FPSstartframe;
	int FPScounted;
	
	//ObjectManage
	NSMutableArray *Objects; //objects
	NSMutableArray *NewbornObjects;
	NSMutableArray *ZombieObjects;
	ArteOGLObject * cameraFocusedObject;
	
	//optimize
	IMP delegateDidCalc, delegateDidDraw;
	
	//mouse
	CGPoint draggingMousePoint; //updated while dragging
	
	float drawColor[4];
}

- (CGPoint *)draggingMousePoint;

- (void)draw;
- (void)OGLScroll;
- (void)OGLHitCheck;
- (void)OGLReap:(ArteOGLObject *)obj;

#pragma mark Genoside
- (void)killObject:(id)obj;
- (void)killThemAll;

- (int) processFlag;
- (void) setProcessFlag:(int)flag;

- (void) OGLprepare;
@end
