
@protocol ArteOGLViewProtocol
#import "ArteOGLDefines.h"

- (void)reset;

#pragma mark Screen Manage

- (CGSize)screenSize;			// size of screen
- (CGRect)screenRect;			// rect of screen

- (void)swichScreenMode;		// switch screen-mode Window<->FullScreen
- (BOOL)isFullScreenMode;		// return YES if fullscreen
- (void)startFullScreen;		// change mode to fullscreen
- (void)endFullScreen;			// change mode to window


#pragma mark Object Manage
- (BOOL)hasObject:(id)obj;				// is object in list
- (BOOL)setObject:(id)obj;				// set object to list
- (BOOL)setObjects:(NSArray *)array;	// set objects to list
- (BOOL)removeObject:(id)obj;			// remove object from list
- (BOOL)removeObjects:(NSArray *)array;	// remove object from list
- (void)removeAllObjects;				// remove all object from list



#pragma mark VirtualCoord

- (CGPoint)virtualDatum;
- (void)resetVirtualCoordOrigin;

- (CGPoint)xyAtVxy:(CGPoint) vx;
- (float)xAtVx:(float) vx;
- (float)yAtVy:(float) vy;
- (CGPoint)vxyAtXY:(CGPoint) vx;
- (float)vxAtX:(float) x;
- (float)vyAtY:(float) y;



#pragma mark Scrolling
- (void)resetScrollMode;
- (ArteOGLScrollMode)horizontalScrollMode;
- (ArteOGLScrollMode)verticalScrollMode;
- (void)setHorizontalScrollMode:(ArteOGLScrollMode)mode;
- (void)setVerticalScrollMode:(ArteOGLScrollMode)mode;
- (CGPoint)scrollSpeed;
- (void)setScrollSpeed:(CGPoint)speeds;
- (void)setScrollEdge:(float)f toPosition:(int)p;	//p:ArteOGLLeft-ArteOGLTop
- (float)scrollEdgeAtPosition:(int)p;				//p:ArteOGLLeft-ArteOGLTop
- (void)setScrollLimit:(float)f toPosition:(int)p;	//p:ArteOGLLeft-ArteOGLTop
- (float)scrollLimitAtPosition:(int)p;				//p:ArteOGLLeft-ArteOGLTop
- (CGPoint)scrollTarget;
- (float)horizontalScrollTarget;
- (float)verticalScrollTarget;
- (void)setScrollTarget:(CGPoint)pt;
- (void)setHorizontalScrollTarget:(float)pt;
- (void)setVerticalScrollTarget:(float)pt;
- (CGPoint)scrollDelay;
- (float)horizontalScrollDelay;
- (float)verticalScrollDelay;
- (void)setScrollDelay:(CGPoint)delay;
- (void)setHorizontalScrollDelay:(float)delay;
- (void)setVerticalScrollDelay:(float)delay;
- (void)setScrollFocusToObject:(id)obj;		//deprecated
- (void)scrollToObject:(id)obj;				//set ScrollTarget to Object's xy.
- (void)forceScrollToObject:(id)obj;		//rewrite VirtualcoordDatum directory.

#pragma mark pause
- (void)togglePause;
- (BOOL)pause;
- (void)setPause:(BOOL)bl;

#pragma mark stage
- (void)setStageBoundToScreenSize;
- (void)setStageBounds:(CGRect)rect;
- (void)setStageBoundsLeft:(float)f;
- (void)setStageBoundsRight:(float)f;
- (void)setStageBoundsBottom:(float)f;
- (void)setStageBoundsTop:(float)f;
- (void)clearStageBounds;

#pragma mark Physic/Collision/MouseSupport
- (id)nearestCollisionObject:(id)obj;
- (NSArray *)collisionsAtRect:(CGRect)rect;
- (id)foremostObjectExistsAtPoint:(CGPoint)pt;
- (NSArray *)objectsExistsAtRect:(CGRect)rect;
- (NSArray *)objectsExistsAtPoint:(CGPoint)pt;
- (NSMutableIndexSet *)indexesOfObjectsExistsAtRect:(CGRect)rect;
- (NSMutableIndexSet *)indexesOfObjectsExistsAtPoint:(CGPoint)pt;

#pragma mark DrawingSupport
- (void)setDrawColorRed:(float)r green:(float)g blue:(float)b alpha:(float)a;
- (void)setDrawColorToRed;

- (void)fillRect:(CGRect)rect;
- (void)frameRect:(CGRect)rect;

- (void)enterRotate:(float)angle withCenter:(CGPoint)pt;
- (void)enterRotate:(float)angle withCenter:(CGPoint)pt offset:(CGPoint)pt2;
- (void)escapeRotate;

- (void)revertBlendMode;
- (void)setBlendMode:(ArteOGLViewBlendMode)mode;

- (long)framecount;

@end

